'use strict';

exports.__esModule = true;

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getColour(r, g, b) {
    var a = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 1;

    if (typeof r === 'string') {
        return r;
    }
    if (typeof r === 'number') {
        return 'rgba(' + r + ',' + b + ',' + g + ',' + a + ')';
    }
    return null;
}

var Graphics = function () {
    function Graphics(width, height) {
        (0, _classCallCheck3.default)(this, Graphics);

        if ((typeof width === 'undefined' ? 'undefined' : (0, _typeof3.default)(width)) === 'object' && width.tagName === 'CANVAS') {
            this.canvas = width;
        } else {
            this.canvas = document.createElement('canvas');
            this.size(width, height);
        }
        this.ctx = this.canvas.getContext('2d');
    }

    Graphics.prototype.fill = function fill(r, g, b) {
        var a = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 1;

        this.ctx.fillStyle = getColour(r, g, b, a);
        return this;
    };

    Graphics.prototype.stroke = function stroke(r, g, b) {
        var a = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 1;

        this.ctx.strokeStyle = getColour(r, g, b, a);
        return this;
    };

    Graphics.prototype.circle = function circle(x, y, radius) {
        var ctx = this.ctx;

        ctx.beginPath();
        ctx.arc(x, y, radius, 0, Math.PI * 2, false);
        ctx.fill();
        return this;
    };

    Graphics.prototype.rect = function rect(x, y, width, height) {
        var angle = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 0;
        var ctx = this.ctx;

        if (angle !== 0) {
            ctx.save();
            ctx.translate(x + width / 2, y + height / 2);
            ctx.rotate(angle);
            ctx.beginPath();
            ctx.rect(-width / 2, -height / 2, width, height);
            ctx.fill();
            ctx.stroke();
            ctx.restore();
        } else {
            ctx.rect(x, y, width, height);
            ctx.fill();
            ctx.stroke();
        }
        return this;
    };

    Graphics.prototype.line = function line(x1, y1, x2, y2) {
        var ctx = this.ctx;

        ctx.beginPath();
        ctx.moveTo(x1, y1);
        ctx.lineTo(x2, y2);
        ctx.stroke();
        return this;
    };

    Graphics.prototype.lineWidth = function lineWidth(width) {
        this.ctx.lineWidth = width;
        return this;
    };

    Graphics.prototype.move = function move(x, y) {
        this.ctx.moveTo(x, y);
        return this;
    };

    Graphics.prototype.image = function image(el, x, y, options) {
        var ctx = this.ctx;

        if (options) {
            var _options$alpha = options.alpha,
                alpha = _options$alpha === undefined ? 1 : _options$alpha,
                _options$rotation = options.rotation,
                rotation = _options$rotation === undefined ? 0 : _options$rotation,
                _options$scale = options.scale,
                scale = _options$scale === undefined ? 1 : _options$scale;

            var offsetX = el.width / 2;
            var offsetY = el.height / 2;
            ctx.save();
            ctx.translate(x + offsetX, y + offsetY);
            ctx.rotate(rotation);
            ctx.scale(scale, scale);
            ctx.globalAlpha = alpha;
            ctx.drawImage(el, -offsetX, -offsetY);
            ctx.restore();
        } else {
            ctx.drawImage(el, x, y);
        }
        return this;
    };

    Graphics.prototype.text = function text(str, x, y) {
        this.ctx.fillText(str, x, y);
        return this;
    };

    Graphics.prototype.setFontStyle = function setFontStyle(family, size) {
        this.ctx.font = size + 'px ' + family;
    };

    Graphics.prototype.getImageData = function getImageData() {
        var x = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
        var y = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
        var width = arguments[2];
        var height = arguments[3];
        var ctx = this.ctx,
            canvas = this.canvas;

        return ctx.getImageData(x, y, width || canvas.width, height || canvas.height);
    };

    Graphics.prototype.getPixel = function getPixel(x, y) {
        x = Math.floor(x);
        y = Math.floor(y);

        var _ctx$getImageData = this.ctx.getImageData(x, y, 1, 1),
            data = _ctx$getImageData.data;

        return Array.prototype.slice.call(data);
    };

    Graphics.prototype.setPixel = function setPixel(x, y, r, g, b, a) {
        x = Math.floor(x);
        y = Math.floor(y);

        var _getImageData = this.getImageData(),
            width = _getImageData.width,
            data = _getImageData.data;

        var i = (x + y * width) * 4;
        data[i + 0] = r;
        data[i + 1] = g;
        data[i + 2] = b;
        data[i + 3] = a;
        return this;
    };

    Graphics.prototype.clearCircle = function clearCircle(x, y) {
        var radius = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 20;
        var ctx = this.ctx;

        ctx.save();
        ctx.globalCompositeOperation = 'destination-out';
        this.circle(x, y, radius);
        ctx.globalCompositeOperation = 'source-over';
        ctx.restore();
        return this;
    };

    Graphics.prototype.translateAnd = function translateAnd(x, y, fn) {
        var ctx = this.ctx;

        ctx.save();
        ctx.translate(x, y);
        fn(ctx);
        ctx.restore();
        return this;
    };

    Graphics.prototype.clear = function clear(r, g, b) {
        var a = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 1;

        var color = getColour(r, g, b, a);
        var ctx = this.ctx;
        var _canvas = this.canvas,
            width = _canvas.width,
            height = _canvas.height;

        ctx.save();
        ctx.setTransform(1, 0, 0, 1, 0, 0);
        if (color) {
            ctx.fillStyle = color;
            ctx.fillRect(0, 0, width, height);
        } else {
            ctx.clearRect(0, 0, width, height);
        }
        ctx.beginPath();
        ctx.restore();
        return this;
    };

    Graphics.prototype.size = function size() {
        var width = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : window.innerWidth;
        var height = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : window.innerHeight;

        this.canvas.width = width;
        this.canvas.height = height;
        return this;
    };

    Graphics.prototype.destroy = function destroy() {
        if (this.canvas.parentElement) {
            this.canvas.parentElement.removeChild(this.canvas);
        }
        this.canvas = null;
        this.ctx = null;
    };

    (0, _createClass3.default)(Graphics, [{
        key: 'alpha',
        get: function get() {
            return this.ctx.globalAlpha;
        },
        set: function set(value) {
            this.ctx.globalAlpha = value;
        }
    }, {
        key: 'blendMode',
        get: function get() {
            return this.ctx.globalCompositeOperation;
        },
        set: function set(value) {
            this.ctx.globalCompositeOperation = value;
        }
    }, {
        key: 'context',
        get: function get() {
            return this.ctx;
        }
    }]);
    return Graphics;
}();

exports.default = Graphics;