'use strict';

exports.__esModule = true;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _loop = require('../loop');

var _loop2 = _interopRequireDefault(_loop);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var dpr = Math.min(2, typeof window !== 'undefined' && (window.devicePixelRatio || 1));

var SpritesheetPlayer = function () {
    function SpritesheetPlayer() {
        var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
            _ref$canvas = _ref.canvas,
            canvas = _ref$canvas === undefined ? typeof document !== 'undefined' && document.createElement('canvas') : _ref$canvas,
            _ref$data = _ref.data,
            data = _ref$data === undefined ? null : _ref$data,
            image = _ref.image,
            _ref$fps = _ref.fps,
            fps = _ref$fps === undefined ? 12 : _ref$fps,
            _ref$trim = _ref.trim,
            trim = _ref$trim === undefined ? true : _ref$trim,
            _ref$delay = _ref.delay,
            delay = _ref$delay === undefined ? 0 : _ref$delay,
            _ref$repeatDelay = _ref.repeatDelay,
            repeatDelay = _ref$repeatDelay === undefined ? 0 : _ref$repeatDelay,
            _ref$scale = _ref.scale,
            scale = _ref$scale === undefined ? 1 : _ref$scale,
            _ref$loop = _ref.loop,
            loop = _ref$loop === undefined ? true : _ref$loop;

        (0, _classCallCheck3.default)(this, SpritesheetPlayer);

        this.canvas = canvas;
        this.data = data;
        this.image = image;
        this.trim = trim;
        this.interval = 1000 / fps;
        this.last = 0;
        this.currentFrame = 0;
        this.playing = false;
        this.delay = delay;
        this.loop = loop;
        this.repeatDelay = repeatDelay;
        this.remainingDelay = delay;
        this.scale = scale * dpr;

        this.update = this.update.bind(this);

        if (data) {
            this.init(data);
        }
    }

    SpritesheetPlayer.prototype.play = function play() {
        if (this.playing) {
            return;
        }
        this.remainingDelay = this.delay;
        this.elapsed = 0;
        _loop2.default.add(this.update);
        _loop2.default.start();
        this.playing = true;
    };

    SpritesheetPlayer.prototype.pause = function pause() {
        _loop2.default.remove(this.looper);
        // loop.stop();
        this.playing = false;
    };

    SpritesheetPlayer.prototype.reset = function reset() {
        this.currentFrame = 0;
        this.remainingDelay = this.delay;
        this.elapsed = 0;
        this.pause();
        this.draw(0);
    };

    SpritesheetPlayer.prototype.updateDimensions = function updateDimensions() {
        this.canvas.width = this.w * this.scale;
        this.canvas.height = this.h * this.scale;
        this.canvas.style.width = this.w * this.scale / dpr + 'px';
        this.canvas.style.height = this.h * this.scale / dpr + 'px';
    };

    SpritesheetPlayer.prototype.setScale = function setScale(value) {
        this.scale = value * dpr;
        this.updateDimensions();
        this.reset();
    };

    SpritesheetPlayer.prototype.init = function init(data) {
        var _this = this;

        var rawFrames = Array.isArray(data.frames) ? data.frames : Object.keys(data.frames).map(function (i) {
            return data.frames[i];
        });
        var trimX = this.trim ? Math.min.apply(Math, rawFrames.map(function (f) {
            return f.spriteSourceSize.x;
        })) : 0;
        var trimY = this.trim ? Math.min.apply(Math, rawFrames.map(function (f) {
            return f.spriteSourceSize.y;
        })) : 0;
        var frames = rawFrames.map(function (f) {
            return Object.assign({}, f.frame, {
                tx: f.spriteSourceSize.x - trimX,
                ty: f.spriteSourceSize.y - trimY
            });
        });

        if (this.trim) {
            this.w = Math.max.apply(Math, frames.map(function (f) {
                return f.tx + f.w;
            }));
            this.h = Math.max.apply(Math, frames.map(function (f) {
                return f.ty + f.h;
            }));
        } else {
            this.w = Math.max.apply(Math, rawFrames.map(function (f) {
                return f.sourceSize.w;
            }));
            this.h = Math.max.apply(Math, rawFrames.map(function (f) {
                return f.sourceSize.h;
            }));
        }

        this.frames = frames;
        this.updateDimensions();
        this.ctx = this.canvas.getContext('2d');

        if (typeof this.image === 'string') {
            var img = new Image();
            img.onload = function () {
                return _this.setImg(img);
            };
            img.src = this.image;
        } else {
            this.setImg(this.image);
        }
    };

    SpritesheetPlayer.prototype.setImg = function setImg(img) {
        this.img = img;
        this.draw(0);
    };

    SpritesheetPlayer.prototype.update = function update(df, dt) {
        this.elapsed += dt;
        if (this.elapsed < this.interval) {
            return;
        }
        this.elapsed = 0;

        if (this.remainingDelay > 0) {
            this.remainingDelay -= this.interval;
            return;
        }

        if (!this.img) {
            return;
        }

        var currentFrame = this.currentFrame + 1;

        if (currentFrame === this.frames.length) {
            if (!this.loop) {
                return;
            }
            this.remainingDelay = this.repeatDelay;
            currentFrame = 0;
        }

        this.draw(currentFrame);

        this.currentFrame = currentFrame;
    };

    SpritesheetPlayer.prototype.draw = function draw(currentFrame) {
        // this.ctx.fillStyle = 'red';
        // this.ctx.fillRect(0, 0, this.canvas.width, this.canvas.height);
        this.ctx.clearRect(0, 0, this.canvas.width, this.canvas.height);

        var frame = this.frames[currentFrame];

        this.ctx.drawImage(this.img, frame.x, frame.y, frame.w, frame.h, frame.tx * this.scale, frame.ty * this.scale, frame.w * this.scale, frame.h * this.scale);
    };

    return SpritesheetPlayer;
}();

exports.default = SpritesheetPlayer;