'use strict';

exports.__esModule = true;
exports.Loop = undefined;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _eventemitter = require('eventemitter3');

var _eventemitter2 = _interopRequireDefault(_eventemitter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Loop = exports.Loop = function () {
    function Loop() {
        (0, _classCallCheck3.default)(this, Loop);

        this.update = this.update.bind(this);

        this.last = 0;
        this.raf = null;
        this.running = false;

        this.emitter = new _eventemitter2.default();
    }

    Loop.prototype.start = function start() {
        if (this.running) {
            return;
        }

        this.last = 0;
        this.running = true;
        this.update();
    };

    Loop.prototype.stop = function stop() {
        if (!this.running) {
            return;
        }

        this.running = false;
        window.cancelAnimationFrame(this.raf);
    };

    Loop.prototype.update = function update() {
        if (!this.running) {
            return;
        }

        this.raf = window.requestAnimationFrame(this.update);

        var now = Date.now();
        var deltaTime = now - this.last;
        var deltaFrames = deltaTime * 0.06;
        this.last = now;

        this.emitter.emit('update', deltaFrames, deltaTime);
    };

    Loop.prototype.add = function add(fn) {
        this.emitter.on('update', fn);
        return this;
    };

    Loop.prototype.remove = function remove(fn) {
        this.emitter.removeListener('update', fn);
    };

    return Loop;
}();

exports.default = new Loop();