'use strict';

exports.__esModule = true;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _linkedList = require('../linked-list');

var _linkedList2 = _interopRequireDefault(_linkedList);

var _objectPool = require('../object-pool');

var _objectPool2 = _interopRequireDefault(_objectPool);

var _index = require('./index');

var _index2 = _interopRequireDefault(_index);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ParticleGroup = function () {
    function ParticleGroup(factoryFn) {
        (0, _classCallCheck3.default)(this, ParticleGroup);

        if (!factoryFn) {
            factoryFn = function factoryFn() {
                return new _index2.default();
            };
        }
        this.update = this.update.bind(this);
        this.list = (0, _linkedList2.default)();
        this.pool = (0, _objectPool2.default)(factoryFn);
    }

    ParticleGroup.prototype.create = function create(options) {
        var p = this.pool.get();
        p.reset(options);
        this.list.add(p);
        return p;
    };

    ParticleGroup.prototype.add = function add(p) {
        this.list.add(p);
    };

    ParticleGroup.prototype.remove = function remove(p) {
        this.list.remove(p);
        this.pool.dispose(p);
    };

    ParticleGroup.prototype.forEach = function forEach(fn) {
        var p = this.list.first;
        while (p) {
            fn(p);
            p = p.next;
        }
    };

    ParticleGroup.prototype.update = function update(fn) {
        var p = this.list.first;
        while (p) {
            var next = p.next;
            p.update();
            if (typeof fn === 'function') {
                fn(p);
            }
            if (!p.alive) {
                this.remove(p);
            }
            p = next;
        }
    };

    return ParticleGroup;
}();

exports.default = ParticleGroup;