"use strict";

exports.__esModule = true;

var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Node = function () {
    function Node(bounds, depth, maxDepth, maxChildren) {
        (0, _classCallCheck3.default)(this, Node);

        this._bounds = bounds;
        this._depth = depth;
        this._maxDepth = maxDepth;
        this._maxChildren = maxChildren;

        this.children = [];
        this.nodes = [];
    }

    Node.prototype.insert = function insert(item) {
        if (this.nodes.length) {
            var index = this._findIndex(item);
            this.nodes[index].insert(item);
            return;
        }

        this.children.push(item);

        if (!(this._depth >= this._maxDepth) && this.children.length > this._maxChildren) {

            this.subdivide();

            for (var i = 0; i < this.children.length; i++) {
                this.insert(this.children[i]);
            }

            this.children.length = 0;
        }
    };

    Node.prototype.retrieve = function retrieve(item) {
        if (this.nodes.length) {
            var index = this._findIndex(item);
            return this.nodes[index].retrieve(item);
        }

        return this.children;
    };

    Node.prototype._findIndex = function _findIndex(item) {
        var _bounds = this._bounds,
            x = _bounds.x,
            y = _bounds.y,
            width = _bounds.width,
            height = _bounds.height;


        var right = item.x > x + width / 2;
        var bottom = item.y > y + height / 2;

        var index = void 0;

        if (right) {
            index = bottom ? Node.BR : Node.TR;
        } else {
            index = bottom ? Node.BL : Node.TL;
        }

        return index;
    };

    Node.prototype.subdivide = function subdivide() {
        var depth = this._depth + 1;

        var _bounds2 = this._bounds,
            x = _bounds2.x,
            y = _bounds2.y,
            width = _bounds2.width,
            height = _bounds2.height;

        var w = width / 2;
        var h = height / 2;

        this.nodes[Node.TL] = new Node({
            x: x,
            y: y,
            width: w,
            height: h
        }, depth, this._maxDepth, this._maxChildren);

        this.nodes[Node.TR] = new Node({
            x: x + w,
            y: y,
            width: w,
            height: h
        }, depth, this._maxDepth, this._maxChildren);

        this.nodes[Node.BL] = new Node({
            x: x,
            y: y + h,
            width: w,
            height: h
        }, depth, this._maxDepth, this._maxChildren);

        this.nodes[Node.BR] = new Node({
            x: x + w,
            y: y + h,
            width: w,
            height: h
        }, depth, this._maxDepth, this._maxChildren);
    };

    Node.prototype.clear = function clear() {
        this.children.length = 0;

        while (this.nodes.length) {
            this.nodes.pop().clear();
        }
    };

    return Node;
}();

Node.TL = 0;
Node.TR = 1;
Node.BL = 2;
Node.BR = 3;

var QuadTree = function () {
    function QuadTree(bounds) {
        var maxDepth = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : -1;
        var maxChildren = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : -1;
        (0, _classCallCheck3.default)(this, QuadTree);

        this.root = new Node(bounds, 0, maxDepth, maxChildren);
    }

    QuadTree.prototype.insert = function insert(item) {
        if (Array.isArray(item)) {
            for (var i = 0; i < item.length; i++) {
                this.root.insert(item[i]);
            }
        } else {
            this.root.insert(item);
        }
    };

    QuadTree.prototype.clear = function clear() {
        this.root.clear();
    };

    QuadTree.prototype.retrieve = function retrieve(item) {
        return this.root.retrieve(item);
    };

    return QuadTree;
}();

exports.default = QuadTree;