// @custom-media --min-viewport (max-width: 639px);
// @custom-media --sm-viewport (min-width: 640px);
// @custom-media --md-viewport (min-width: 960px);
// @custom-media --lg-viewport (min-width: 1200px);
// @custom-media --xlg-viewport (min-width: 1440px);
import eventBus from './eventBus';

const breakpoints = {
    min: 0,
    sm: 768,
    md: 1024,
    lg: 1160,
    xlg: 1440,
    xxlg: 1700
};

let currentSize = null, el = null;

function showDebugInfo() {
    if (!el) {
        el = document.createElement('pre');
        el.style.position = 'fixed';
        el.style.zIndex = '99999';
        el.style.right = '0';
        el.style.bottom = '0';
        el.style.color = 'black';
        el.style.fontSize = '12px';
        el.style.padding = '2px 10px';
        el.style.background = 'white';
        el.style.transition = 'background 0.3s';
        document.body.appendChild(el);
    }

    el.innerHTML = `${window.innerWidth} breakpoint: ${currentSize} (${breakpoints[currentSize]})`;
    el.style.background = 'red';
    window.setTimeout(() => (el.style.background = 'white'), 600);
}

function update() {
    const width = window.innerWidth;
    const newSize = Object.keys(breakpoints).reduce((value, bp) => {
        if (breakpoints[bp] <= width) {
            value = bp;
        }
        return value;
    }, 'sm');

    if (newSize !== currentSize) {
        currentSize = newSize;
        console.debug('breakpoint', currentSize, width);
        // showDebugInfo();
        eventBus.emit('breakpoint', currentSize);
    }
}

export default function breakpoint() {

    if (!currentSize) {
        update();
        eventBus.on('resize', update);
    }

    return currentSize;
}

breakpoint.is = function(size) {
    return breakpoint() === size;
};

breakpoint.lt = function(size) {
    return breakpoints[breakpoint()] < breakpoints[size];
};

breakpoint.gt = function(size) {
    return breakpoints[breakpoint()] > breakpoints[size];
};
