'use strict';

// app dependencies
var Utils = require('./utils');

// define
var Detector = {};

// ------------------------------------------------------------------------
// device and browser
// ------------------------------------------------------------------------

// shortcut for user agent
var ua = (navigator.userAgent || navigator.vendor || window.opera);

// ios
Detector.isIpad = !!ua.match(/iPad/i);
Detector.isIphone = !!ua.match(/iPhone/i);
Detector.isIpod = !!ua.match(/iPod/i);
Detector.isIOS = (Detector.isIpad || Detector.isIphone || Detector.isIpod);
Detector.isIOS5 = !!(ua.match(/OS 5(_\d)+ like Mac OS X/i));
Detector.isIOS6 = !!(ua.match(/OS 6(_\d)+ like Mac OS X/i));
Detector.isIOS7 = !!(ua.match(/OS 7(_\d)+ like Mac OS X/i));
Detector.isIOS8 = !!(ua.match(/OS 8(_\d)+ like Mac OS X/i));

// Detector.IOSVersion = (function() {
//     if (/iP(hone|od|ad)/.test(navigator.platform)) {
//         // supports iOS 2.0 and later: <http://bit.ly/TJjs1V>
//         var v = (navigator.appVersion).match(/OS (\d+)_(\d+)_?(\d+)?/);
//         console.log('HERE :', v);
//         return parseInt(v[1], 10); // [parseInt(v[1], 10), parseInt(v[2], 10), parseInt(v[3] || 0, 10)];
//     } else {
//         return false;
//     }
// })();

Detector.IOSVersion = (function() {
    if (/iP[ao]d|iPhone/i.test(ua)) {
        var matches = ua.match(/OS (\d+)_(\d+)/i);
        if (matches && matches.length > 2) {
            return parseFloat(matches[1] + '.' + matches[2]);
        }
    }
    return -1;
}());

// android
Detector.isAndroid = !!ua.match(/Android/i);
Detector.isAndroidOld = !!(Detector.isAndroid && parseFloat(ua.slice(ua.indexOf('Android') + 8)) < 4);
Detector.isAndroidStock = (function() {
    var regExAppleWebKit = new RegExp(/AppleWebKit\/([\d.]+)/);
    var resultAppleWebKitRegEx = regExAppleWebKit.exec(ua);
    var appleWebKitVersion = (resultAppleWebKitRegEx === null ? null : parseFloat(regExAppleWebKit.exec(ua)[1]));
    var isAndroidBrowser = Detector.isAndroid && appleWebKitVersion !== null && appleWebKitVersion < 537;
    return isAndroidBrowser;
})();

// device
Detector.isTablet = (Detector.isAndroid || Detector.isIOS) && (/ipad|android 3|sch-i800|playbook|tablet|kindle|gt-p1000|sgh-t849|shw-m180s|a510|a511|a100|dell streak|silk/i.test(ua.toLowerCase()));
Detector.isMobile = (Detector.isAndroid || Detector.isIOS) && (/iphone|ipod|android|blackberry|opera mini|opera mobi|skyfire|maemo|windows phone|palm|iemobile|symbian|symbianos|fennec/i.test(ua.toLowerCase()));
Detector.isDesktop = (!Detector.isTablet && !Detector.isMobile);
Detector.device = Detector.isTablet ? 'tablet' : Detector.isMobile ? 'mobile' : 'desktop';

// ie
Detector.IEVersion = (function() {
    var rv = -1,
        re;
    if (navigator.appName === 'Microsoft Internet Explorer') {
        re = new RegExp('MSIE ([0-9]{1,}[.0-9]{0,})');
        if (re.exec(ua) !== null) {
            rv = parseFloat(RegExp.$1);
        }
    } else if (navigator.appName === 'Netscape') {
        re = new RegExp('Trident/.*rv:([0-9]{1,}[.0-9]{0,})');
        if (re.exec(ua) !== null) {
            rv = parseFloat(RegExp.$1);
        }
    }
    return rv;
})();
Detector.isIE = (Detector.IEVersion > -1);
Detector.isIE11 = (Detector.IEVersion === 11);
Detector.isIEOld = (function() {
    var div = document.createElement('div');
    div.innerHTML = '<!--[if IE]><i></i><![endif]-->';
    return (div.getElementsByTagName('i').length === 1);
})();
Detector.isIE8Down = (function() {
    var div = document.createElement('div');
    div.innerHTML = '<!--[if lte IE 8]><i></i><![endif]-->';
    return (div.getElementsByTagName('i').length === 1);
})();
Detector.isIE9Down = (function() {
    var div = document.createElement('div');
    div.innerHTML = '<!--[if lte IE 9]><i></i><![endif]-->';
    return (div.getElementsByTagName('i').length === 1);
})();

// firefox
Detector.isFirefox = (window.mozInnerScreenX !== null);

// chrome
Detector.isChrome = (function() {
    var isChromium = window.chrome,
        vendorName = window.navigator.vendor;
    return (isChromium !== null && isChromium !== undefined && vendorName === 'Google Inc.');
})();

// ------------------------------------------------------------------------
// interaction and dom/css
// ------------------------------------------------------------------------

// interaction
Detector.hasHistory = !!(history && history.pushState);
Detector.hasMouseMove = Detector.isDesktop;
Detector.hasTouch = ('ontouchstart' in window || 'onmsgesturechange' in window);
Detector.hasFullscreen = (function() {
    var element = document.body;
    return !!(element.requestFullScreen || element.webkitRequestFullScreen || element.mozRequestFullScreen || element.msRequestFullscreen);
})();
Detector.supportsMagipack = !Detector.isIE8Down;
// dom/css
Detector.transitionEnd = (function() {
    var t,
        el = document.createElement('fakeelement'),
        transitions = {
            transition: 'transitionend',
            OTransition: 'oTransitionEnd',
            MozTransition: 'transitionend',
            WebkitTransition: 'webkitTransitionEnd'
        },
        transitionend;
    for (t in transitions) {
        if (el.style[t] !== undefined) {
            transitionend = transitions[t];
            break;
        }
    }
    return transitionend;
})();
Detector.visibilityChangeEventName = undefined;
Detector.hiddenPropertyName = undefined;
if (typeof document.hidden !== 'undefined') { // Opera 12.10 and Firefox 18 and later support
    Detector.hiddenPropertyName = 'hidden';
    Detector.visibilityChangeEventName = 'visibilitychange';
} else if (typeof document.mozHidden !== 'undefined') {
    Detector.hiddenPropertyName = 'mozHidden';
    Detector.visibilityChangeEventName = 'mozvisibilitychange';
} else if (typeof document.msHidden !== 'undefined') {
    Detector.hiddenPropertyName = 'msHidden';
    Detector.visibilityChangeEventName = 'msvisibilitychange';
} else if (typeof document.webkitHidden !== 'undefined') {
    Detector.hiddenPropertyName = 'webkitHidden';
    Detector.visibilityChangeEventName = 'webkitvisibilitychange';
}

// ------------------------------------------------------------------------
// pixel ratio
// ------------------------------------------------------------------------

Detector.pixelRatio = window.devicePixelRatio || 1;
Detector.isRetina = (Detector.pixelRatio > 1);

// ------------------------------------------------------------------------
// media
// ------------------------------------------------------------------------

// Detector.noVideoPreload = Detector.isIphone || Detector.isIpod || Detector.isIOS5 || Detector.isAndroidOld;
// Detector.singleVideoElement = Detector.isIos || Detector.isAndroid;
// Detector.singleVideoPreload = (Detector.isIpad && !Detector.isIOS5) || Detector.isAndroid;
// Detector.videoExtension = Modernizr.video.webm ? 'webm' : Modernizr.video.h264 ? 'm4v' : 'ogv';
Detector.videoExtension = Modernizr.video.webm ? 'webm' : Modernizr.video.h264 ? 'mp4' : 'ogv';
Detector.audioExtension = (Modernizr.audio.ogg > Modernizr.audio.mp3) ? 'ogg' : 'mp3';
Detector.isHd = Detector.isDesktop;
Detector.res = Detector.isHd ? 'hd' : 'sd';

Detector.webp = Detector.isChrome;
Modernizr.addTest('webp', function() {
    return Detector.webp;
});

// ------------------------------------------------------------------------
// version
// ------------------------------------------------------------------------

Detector.isGold = false;
Detector.isSilver = false;
Detector.isBronze = false;
Detector.isStatic = false;
if (Detector.isDesktop) {
    // desktop
    if (Modernizr.webgl && Modernizr.video) {
        Detector.isGold = true;
    } else if (Modernizr.canvas && Modernizr.video && (!Detector.isIE || Detector.IEVersion >= 10)) {
        Detector.isSilver = true;
    } else {
        Detector.isStatic = true;
    }
} else if (Detector.isTablet) {
    // tablet
    if (Detector.isIOS && Detector.IOSVersion >= 7) {
        Detector.isSilver = true;
    } else if (Detector.isIOS && Detector.IOSVersion >= 5) {
        Detector.isBronze = true;
    } else if (Detector.isAndroid && !Detector.isAndroidOld) {
        Detector.isBronze = true;
    } else {
        Detector.isStatic = true;
    }
} else {
    // mobile
    if (Detector.isIOS && Detector.IOSVersion >= 5) {
        Detector.isBronze = true;
    } else if (Detector.isAndroid && !Detector.isAndroidOld) {
        Detector.isBronze = true;
    } else {
        Detector.isStatic = true;
    }
}
Detector.experience = Detector.isGold ? 'gold' : Detector.isSilver ? 'silver' : Detector.isBronze ? 'bronze' : 'static';

// override experience properties by query string value
var params = Utils.getQueryString();
if (params && params.experience) {
    Detector.experience = params.experience;
    switch (Detector.experience) {
        case 'gold':
            Detector.isGold = true;
            Detector.isSilver = false;
            Detector.isBronze = false;
            Detector.isStatic = false;
            break;
        case 'silver':
            Detector.isGold = false;
            Detector.isSilver = true;
            Detector.isBronze = false;
            Detector.isStatic = false;
            break;
        case 'bronze':
            Detector.isGold = false;
            Detector.isSilver = false;
            Detector.isBronze = true;
            Detector.isStatic = false;
            break;
        case 'static':
            Detector.isGold = false;
            Detector.isSilver = false;
            Detector.isBronze = false;
            Detector.isStatic = true;
            break;
    }
}

// Object.keys(Detector).forEach(function(key) { console.debug(key, ' = ', Detector[key]); });

// export
module.exports = Detector;
