'use strict';

module.exports = (function() {

  var id = 0;

  var list = [];

  var time = 0;

  var isRunning = false;

  var update = function() {

    if (!isRunning) { return; }

    window.requestAnimationFrame(update);

    var now = Date.now();

    var delta = now - time;

    time = now;

    list.forEach(function(item) {
      if (delta >= item.interval) {
        item.fn.call(item.context, delta);
      }
    });

  };

  var start = function() {
    isRunning = true;
    time = Date.now();
    update();
  };

  var stop = function() {
    isRunning = false;
  };

  var add = function(fn, context, interval) {
    id++;
    console.log('Raf.add:', id, list.length);
    list.push({
      id: id,
      fn: fn,
      context: context || fn,
      interval: interval || 0
    });
    if (!isRunning) {
      start();
    }
    return id;
  };

  var remove = function(id) {
    if (!id) { return; }
    console.log('Raf.remove:', id);
    list.some(function(item, index) {
      if (item.id === id) {
        return list.splice(index, 1);
      }
    });
  };

  return Object.freeze({
    start: start,
    stop: stop,
    add: add,
    remove: remove
  });

}());
