import eventBus from './eventBus';

function isVisible(item) {
    return item.el.classList.contains('is-visible');
}

function reset(items) {
    items = items.filter((item) => !isVisible(item));
    items.forEach((item) => {
        const rect = item.el.getBoundingClientRect();
        const {height, top} = rect;
        const y = top + (window.pageYOffset || document.documentElement.scrollTop);
        item.y = y;
        item.height = height;
    });
    return items;
}

function show(item) {
    item.el.classList.add('is-visible');
}

function showItems(items, delay, stagger) {
    if (!delay && !stagger) {
        items.forEach(show);
    } else {
        items.forEach((item) => {
            window.setTimeout(() => show(item), delay);
            delay += stagger;
        });
    }
}

function update(items, lastScrollY, threshold, delay = 0, stagger = 0) {
    const height = window.innerHeight;
    const visibles = items.filter((item) => (lastScrollY + height * threshold > item.y));
    showItems(visibles, delay, stagger);
}

export default function(options) {
    const {selector, threshold} = options;
    // const threshold = 0.9;

    // options: threshold, delay, stagger, visible class

    const els = Array.from(document.querySelectorAll(selector));

    if (!els.length) {
        return;
    }

    let items = els.map((el) => ({el}));

    reset(items);

    update(items, 0, threshold, 800, 100);

    eventBus.on('scroll', (position) => update(items, position, threshold, 0, 100));

    eventBus.on('scrollend', () => {
        items = reset(items);
    });

    eventBus.on('resize', () => {
        items = reset(items);
    });
}
