var clone = require('underscore').clone;
var EventEmitter = require('./emitter');

module.exports = function(el, minDistance) {
    el = el || document.body;
    minDistance = minDistance || 10;

    var touchData = {
        start: -1,
        move: -1,
        end: -1,
        distance: 0,
        direction: 'none',
        touching: false,
        originalEvent: null
    };

    var touchX = clone(touchData);
    var touchY = clone(touchData);

    function touchHandler(event) {
        if (!(event && event.touches)) {
            return;
        }
        var touch = event.touches[0];
        var x = touch && touch.pageX;
        var y = touch && touch.pageY;
        switch (event.type) {
            case 'touchstart':
                touchX.start = touchX.move = touchX.end = x;
                touchY.start = touchY.move = touchY.end = y;
                touchX.touching = touchY.touching = true;
                break;
            case 'touchmove':
                touchX.move = x;
                touchY.move = y;
                break;
            case 'touchend':
                touchX.originalEvent = touchY.originalEvent = event;
                update(touchX, 'left', 'right');
                update(touchY, 'up', 'down');
                break;
            default:
        }
    }

    function update(t, a, b) {
        t.end = t.move;
        t.touching = false;
        t.distance = Math.abs(t.start - t.move);
        if (t.distance >= minDistance) {
            t.direction = t.start > t.move ? a : b;
            swipe.emit(t.direction, clone(t));
        }
    }

    function listen(elem) {
        el = elem || el;
        el.addEventListener('touchstart', touchHandler);
        el.addEventListener('touchmove', touchHandler);
        el.addEventListener('touchend', touchHandler);
        return swipe;
    }

    function destroy() {
        swipe.removeAllListeners();
        el.removeEventListener('touchstart', touchHandler);
        el.removeEventListener('touchmove', touchHandler);
        el.removeEventListener('touchend', touchHandler);
        el = null;
        return swipe;
    }

    var swipe = Object.create(EventEmitter.prototype, {
        _events: {
            value: {}
        },
        listen: {
            value: listen
        },
        destroy: {
            value: destroy
        }
    });

    return Object.freeze(swipe);
};
