import emitter from '../events/emitter';

export default function wheel() {

    let enabled = true;

    function getWheelDelta(event) {
        if (event.wheelDelta) {
            return event.wheelDelta / 40;
        }
        return 0 - (event.detail || 0);
    }

    function scrollWheel(event) {
        if (!enabled) {
            return;
        }

        event.preventDefault();

        const delta = getWheelDelta(event);
        if (!delta) {
            return;
        }

        if (delta > 0) {
            wheel.emit('up', delta);
        } else {
            wheel.emit('down', delta);
        }

        wheel.emit('update', delta);
    }

    function setEnabled(value) {
        enabled = value;
    }

    function destroy() {
        wheel.off();
        document.removeEventListener('DOMMouseScroll', scrollWheel);
        document.removeEventListener('mousewheel', scrollWheel);
    }

    document.addEventListener('DOMMouseScroll', scrollWheel, false);
    document.addEventListener('mousewheel', scrollWheel, false);

    wheel = Object.assign(Object.create(emitter.prototype, {
        setEnabled,
        destroy
    });

    return wheel;
};
