'use strict';

exports.__esModule = true;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _quad = require('../ease/quad');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Tween = function () {
    function Tween(ob, props, duration, options) {
        (0, _classCallCheck3.default)(this, Tween);

        this.ob = ob;

        if (props) {
            this.to(props, duration, options);
        }
    }

    Tween.prototype.to = function to(props, duration) {
        var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

        this.duration = duration;
        this.ease = options.ease || _quad.easeOutQuad;
        this.delay = options.delay || 0;
        this.onUpdate = options.onUpdate;
        this.onComplete = options.onComplete;
        this.time = 0;
        this.complete = false;

        this._props = Object.keys(props);
        this._beginVals = {};
        this._changeVals = {};

        for (var i = 0; i < this._props.length; i++) {
            var prop = this._props[i];
            this._beginVals[prop] = this.ob[prop];
            this._changeVals[prop] = props[prop] - this._beginVals[prop];
        }
    };

    Tween.prototype.update = function update(dt) {
        if (this.time === this.duration) {
            return;
        }

        if (this.delay > 0) {
            this.delay -= dt;
            return;
        }

        this.time += dt;

        if (this.time > this.duration) {
            this.time = this.duration;
        }

        for (var i = 0; i < this._props.length; i++) {
            var prop = this._props[i];
            this.ob[prop] = this.ease(this.time, this._beginVals[prop], this._changeVals[prop], this.duration);
        }

        if (this.onUpdate) {
            this.onUpdate(this.ob);
        }

        if (this.time === this.duration) {
            this.complete = true;

            if (this.onComplete) {
                this.onComplete(this.ob);
            }
        }
    };

    Tween.prototype.reset = function reset() {
        this.time = 0;
        this.complete = false;
    };

    return Tween;
}();

exports.default = Tween;